#  ==========================================================================
#  normalize_toolkit_wip.rb
#  --------------------------------------------------------------------------
#
#  Description	:	Intended design of the script is to find manifold faces and
#					then normalize the faces depending on some parameters
#
#  Definition Of Use
#  -----------------
#
#  (1)	Test Manifold Algorithm
#
#		First select a group of edges, then select the menu 'Tools' ->
#		'Normalize Toolkit' -> 'Test manifold algorithm'
#
#  --------------------------------------------------------------------------
#  ==========================================================================
#  --------------------------------------------------------------------------
#
# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#  --------------------------------------------------------------------------
#  ==========================================================================

module NormalizeToolkit


	class NormalizeFaces
		@@cursor_active = 0
		@@entityID = false
		@@tw = Sketchup.create_texture_writer
		@@t1 = 0
		@@t2 = 0
		@@t3 = 0
		@@t4 = 0
		@@t5 = 0
		@@t6 = 0
		
		@@vt_faces_in_surface_length		= "Waiting for input"
		@@vt_border_edge_length				= "..."
		@@vt_non_manifold_faces_length		= "..."
		@@vt_time_taken						= "..."
		def initialize
			#first lets check if we've got a selection of faces
			selection = Sketchup.active_model.selection
			if selection.length <= 0
				self.alert_error("Error: No active selection. Please select the faces you want to normalize first and then run Normalize Surface.",true)
				Sketchup.send_action CMD_SELECT
				return false
			end #if
			
			entity_hash = self.get_entity_hash(selection)
			selection.clear
			@@faces_to_edges = entity_hash["faces_to_edges"]
			@@edges_to_faces = entity_hash["edges_to_faces"]
			@@faces_to_verts = entity_hash["faces_to_verts"]
			@@edges_to_verts = entity_hash["edges_to_verts"]
			if @@faces_to_edges.length <= 0
				self.alert_error("Error: No faces in selection. Please select the faces you want to normalize first and then run Normalize Surface.",true)
				Sketchup.send_action CMD_SELECT
				return false
			end #if
			
			# selection.clear
			# @@face_objects.each {|faceID, face|
				# selection.add face
			# }

			#now we load variables for the single face selection
			@@ph	= Sketchup.active_model.active_view.pick_helper
			self.load_cursors
		end #def
		
		def load_cursors
			@@cursor_specific	= UI::create_cursor((File.join(File.dirname(__FILE__),"normalize_toolkit","cursors","select_specific.png")),4,9)
			@@cursor_true		= UI::create_cursor((File.join(File.dirname(__FILE__),"normalize_toolkit","cursors","select_true.png")),4,9)
			@@cursor_false		= UI::create_cursor((File.join(File.dirname(__FILE__),"normalize_toolkit","cursors","select_false.png")),4,9)
			@@cursor_active = @@cursor_specific
		end #def
		
		def alert_error(msg,critical=false,detail=false)
			puts ((detail != false) ? detail : msg)
			Sketchup.set_status_text msg, SB_PROMPT
			UI::messagebox(msg,MB_OK) if critical
		end #def
		
		def get_entity_hash(selection)
			faces_to_edges = Hash.new()							#these are associative arrays that connect types of
			faces_to_verts = Hash.new()							#entities by their physical connections
			edges_to_verts = Hash.new()
			edges_to_faces = Hash.new()
			@@face_objects = Hash.new()
			@@edge_objects = Hash.new()
			@@vert_objects = Hash.new()
			for i in 0...selection.length
				if selection[i].kind_of?(Sketchup::Face)
					@@face_objects[selection[i].entityID] = selection[i]
					faces_to_edges[selection[i].entityID] = Array.new()
					edges = selection[i].edges
					for j in 0...edges.length
						if !edges_to_verts.has_key?(edges[j].entityID)
							edges_to_verts[edges[j].entityID] = Hash.new()
							verts = edges[j].vertices
							for k in 0...verts.length
								@@vert_objects[verts[k].entityID] = verts[k]
								edges_to_verts[edges[j].entityID][verts[k].entityID] = verts[k]
							end #for
						end #if
						@@edge_objects[edges[j].entityID] = edges[j]
						if !edges_to_faces.has_key?(edges[j].entityID)
							edges_to_faces[edges[j].entityID] = Array.new()
							faces = edges[j].faces
							for k in 0...faces.length
								edges_to_faces[edges[j].entityID] << faces[k].entityID
							end #for
						end #if
						faces_to_edges[selection[i].entityID] << edges[j].entityID
					end #for
					faces_to_verts[selection[i].entityID] = Array.new()
					verts = selection[i].vertices
					for j in 0...verts.length
						@@vert_objects[verts[j].entityID] = verts[j]
						faces_to_verts[selection[i].entityID] << verts[j].entityID
					end #for
				end #if
			end #for
			return Hash[
				"faces_to_edges" => faces_to_edges,
				"edges_to_faces" => edges_to_faces,
				"faces_to_verts" => faces_to_verts,
				"edges_to_verts" => edges_to_verts
			]
		end #def
		
		def onMouseMove(flags, x, y, view)
			selection = Sketchup.active_model.selection
			@@ph.do_pick x, y
			all_entities = @@ph.all_picked
			
			if(all_entities.length > 0)
				@@ph.do_pick x, y
				entity = @@ph.picked_face

				if entity != nil
					selection.clear if @@entityID != entity.entityID
					if @@faces_to_edges.has_key?(entity.entityID)
						@@cursor_active = @@cursor_true
						view.tooltip = "Face in selection"
						if entity.entityID != @@entityID
							selection.add(entity)
							self.refine_surface_conditions(entity,true)
							@@entityID = entity.entityID
						end #if
					else
						@@vt_faces_in_surface_length		= "Waiting for input"
						@@vt_border_edge_length				= "..."
						@@vt_non_manifold_faces_length		= "..."
						@@vt_time_taken						= "..."
						@@cursor_active = @@cursor_false
						view.tooltip = "Face not in selection"
					end
				else
					@@vt_faces_in_surface_length		= "Waiting for input"
					@@vt_border_edge_length				= "..."
					@@vt_non_manifold_faces_length		= "..."
					@@vt_time_taken						= "..."
					selection.clear
					@@cursor_active = @@cursor_false
					view.tooltip = "No Face Found"
					@@entityID = false
				end
			else
				@@vt_faces_in_surface_length		= "Waiting for input"
				@@vt_border_edge_length				= "..."
				@@vt_non_manifold_faces_length		= "..."
				@@vt_time_taken						= "..."
				selection.clear
				@@cursor_active = @@cursor_specific
				view.tooltip = "No Face Found"
				@@entityID = false
			end #if
			
			view.invalidate
		end #def
		
		def refine_surface_conditions(face,update_ui=false)
			faces_to_check 	= Hash[face.entityID => face]		#current pass faces
			surface_group 	= Hash[face.entityID => face]		#hash to store manifold faces for each pass
			unchecked_faces = @@faces_to_edges.clone			#duplicate the selected faces and associated edges hash
			unchecked_edges = @@edges_to_faces.clone			#so we can remove checked faces and edges on each pass
			checked_faces 	= Hash.new()

			border_edges = Hash.new()							#hash to store & revise the border edges
			non_manifold_faces = Hash.new()						#hash to store non-manifold faces
			non_manifold_edges = Hash.new()						#hash to store non-manifold edges
			
			@@vt_faces_in_surface_length		= "Calculating..."
			@@vt_border_edge_length				= "Calculating..."
			@@vt_non_manifold_faces_length		= "Calculating..."
			@@vt_time_taken						= "Calculating..."
			
			time = Time.now() if update_ui
			
			found_border_edges = false
			while !found_border_edges
				found_border_edges = true
				faces_to_check.each {|faceID, face|
					edges_array = unchecked_faces[faceID]
					next if !edges_array
					for i in 0...edges_array.length
						if unchecked_edges.has_key?(edges_array[i])
							faces_array = unchecked_edges[edges_array[i]]				#get the faces associated with the edge find the manifold face
							
							manifold_hash = self.find_manifold_face(faceID,edges_array[i],@@edges_to_faces[edges_array[i]],true)
							
							if !manifold_hash
								border_edges[edges_array[i]] = @@edge_objects[edges_array[i]]			#probably found a border edge!!! excellent
							else
								manifold_face	= manifold_hash["manifold_face"]
								oriented		= manifold_hash["oriented"]
								if !oriented															#found a non-manifold face inside the borders of the selection
									# puts "found non oriented face"
									non_manifold_faces[manifold_face.entityID] = manifold_face			#will need to delete at the end of the pass
									ne_array = unchecked_faces[manifold_face.entityID].clone			#we'll also need to remove the faces borders
									ne_array.delete(edges_array[i])										#from the final border edges
									for j in 0...ne_array.length
										non_manifold_edges[ne_array[j]] = @@edge_objects[ne_array[j]]
									end #for
								else
									surface_group[manifold_face.entityID] = manifold_face				#we've got a manifold face, store it, add it to
									if !checked_faces.has_key?(manifold_face.entityID)
										faces_to_check[manifold_face.entityID] = manifold_face			#faces to check. All other faces associated with
									end #if
									nf_array = faces_array.clone										#the edge are assumed non-manifold so add them to
									nf_array.delete(manifold_face.entityID)								#the non-manifold faces hash.
									# for j in 0...nf_array.length
										# non_manifold_faces[nf_array[j]] = @@face_objects[nf_array[j]]
									# end #for
									non_manifold_edges[edges_array[i]] = @@edge_objects[edges_array[i]]
								end
							end #if
							
							#unchecked_edges.delete(edges_array[i])
						end #if
					end #for
					#unchecked_faces.delete(faceID)
					checked_faces[faceID] = true
					faces_to_check.delete(faceID)
					found_border_edges = false
				}
			end #while
			
			if(surface_group.length > 1)
				non_manifold_faces.each_key {|faceID|	surface_group.delete(faceID);}		#remove all non-manifold faces from the surface group
				non_manifold_edges.each_key {|edgeID|	border_edges.delete(edgeID);}		#remove all non-manifold edges from the border edges
				
				if update_ui
					@@vt_faces_in_surface_length		= surface_group.length.to_s + " face(s)"
					@@vt_border_edge_length				= border_edges.length.to_s + " edge(s)"
					@@vt_non_manifold_faces_length		= non_manifold_faces.length.to_s + " face(s)"
					total_time = (((Time.now() - time)*1000).round).quo(1000)
					@@vt_time_taken						= total_time.to_s + " seconds"
					# puts "time taken: "+(Time.now() - time).to_s
					# puts "border length: "+border_edges.length.to_s
					# puts "surface length: "+surface_group.length.to_s
					# puts "t1: "+@@t1.to_s
					# puts "t2: "+@@t2.to_s
					# puts "t3: "+@@t3.to_s
					# puts "t4: "+@@t4.to_s
					# puts "t5: "+@@t5.to_s
					# puts "t6: "+@@t6.to_s
					selection = Sketchup.active_model.selection
					selection.clear
					selection.add(border_edges.values)
					selection.add(non_manifold_faces.values)
				end #if
				
				return Hash[
					"surface_group"	=> surface_group,
					"border_edges"	=> border_edges,
					"non_manifold_faces"	=> non_manifold_faces,
					"non_manifold_edges"	=> non_manifold_edges
				]
			else
				puts "Error: Unable to determine 2 dimensional manifold - only one face found"
				return false
			end #if
			
		end #def
		
		def intersect_line_line_segment_2d(line,segment)
			x1	= line[0].x
			y1	= line[0].y
			x2	= line[1].x
			y2	= line[1].y
			x3	= segment[0].x
			y3	= segment[0].y
			x4	= segment[1].x
			y4	= segment[1].y
			
			u_line_n = ((x4 - x3)*(y1 - y3)) - ((y4 - y3)*(x1 - x3))	#line equation numerator
			denom	 = ((y4 - y3)*(x2 - x1)) - ((x4 - x3)*(y2 - y1))	#line  & segment equation denominator
			
			u_segment_n	= ((x2 - x1)*(y1 - y3)) - ((y2 - y1)*(x1 - x3))	#segmnet equation numerator
			
			if denom == 0
				#line and segment are parallel
				return false
			end #if
			
			u_segment = u_segment_n.quo(denom)
			
			if u_segment >= 0 || u_segment <=1
				#return [x1 + (u_segment*(x2-x1)),y1 + (u_segment*(y2-y1)),1]	#use to return the point of intersection
				u_line = u_line_n.quo(denom)									#use to return a relative distance value for ordering
				return u_line													#intersections on a complex polygon
			else
				#intersection is outside the line segment
				return false
			end
		end #def
		
		def find_manifold_face(origin_face_id,fold_edge_id,possible_face_ids,check_orientation=false,correct_orientation=false)		
			origin_face		= @@face_objects[origin_face_id]
			origin_edges	= @@faces_to_edges[origin_face_id]
			manifold_face	= false
			
			time = Time.now()
			
			#	PT1: ORIGIN FACE->FOLD EDGE ORIENTATION
			#	we need to find on which side of the edge the face is, and then
			#	determine the direction of rotation. This is more difficult than
			#	it appears. The most error-free way to do it is to create a line
			#	that represents the planar normal for the edge (ie perpendicular
			#	to the fold edge and on the plane of the origin face) and find the
			#	line segments that intersect with the face using an Intersect Segment
			#	with Polygon algorithm. [see: http://softsurfer.com/Archive/algorithm_0111/algorithm_0111.htm]
			#	A COUPLE OF NOTES FIRST:
			# 1:We only need a 2d coordinate system for this so we can use the UVHelper
			#	to generate this and hopefully make the code more efficient
			# 2:We only need to find the first intersected line segment that shares
			#	one of its points with the fold edge
			fold_verts 		= @@edges_to_verts[fold_edge_id]					#construct points on the fold edge that will help with
			fold_arr		= fold_verts.values									#orientation: pt1: start, pt2: middle, pt3: end
			fold_pt1		= fold_arr[0].position
			fold_pt3		= fold_arr[1].position
			fold_pt2		= fold_pt1 + Geom::Vector3d.new((fold_pt3.x-fold_pt1.x).quo(2),(fold_pt3.y-fold_pt1.y).quo(2),(fold_pt3.z-fold_pt1.z).quo(2))
			fold_vec		= fold_pt1.vector_to fold_pt3
			trans 			= Geom::Transformation.rotation fold_pt2, origin_face.normal, -90.degrees
			o_vec1 			= fold_vec.transform trans
			
			origin_uvhelp 	= origin_face.get_UVHelper true, false, @@tw		#UVHelper can give us 2d coordinates for a 3d face :)
			fold_uv_pt1		= origin_uvhelp.get_front_UVQ(fold_pt1)				#construct the corresponding uv points that are uv
			fold_uv_pt2		= origin_uvhelp.get_front_UVQ(fold_pt2)				#representations of the 3d coordinates
			fold_uv_pt3		= origin_uvhelp.get_front_UVQ(fold_pt3)
			fold_uv_vec		= fold_uv_pt1.vector_to fold_uv_pt3
			
			trans_uv_vec	= Geom::Vector3d.new(fold_uv_vec[1],-fold_uv_vec[0],1)		#construct a fold edge normal line to
			trans_uv_pt		= fold_uv_pt2 + trans_uv_vec								#calculate the line segments of intersection
			trans_uv_line	= [fold_uv_pt2, trans_uv_pt]
			
			#Mod 1: classify point mod
			#This works by testing a point thats 0.001 inches from the center of the
			#fold edge (Sketchup's minimum distance). If the model contains erroneous data
			#(faces with sides of sides around the 0.001 inches length) then its possible
			#that this method wont find the side that the face is on respective to the edge
			#or return some weird data. Its more likely this will happen when faces are
			#complex polygons, so it would probably be a good idea to have an option to switch
			#this on and off.
			@@t1 += (Time.now() - time)
			intersection_array = Array[0]
			simple_vec = o_vec1.clone
			simple_vec.length = 0.005
			simple_trans_pt = fold_pt2 + simple_vec
			found_simple_pt = false
			reverse = 1
			if origin_face.classify_point(simple_trans_pt) == Sketchup::Face::PointOutside
				simple_vec.reverse!
				simple_trans_pt = fold_pt2 + simple_vec
				if origin_face.classify_point(simple_trans_pt) != Sketchup::Face::PointOutside
					reverse = -1
					found_simple_pt = true
					simple_vec.length = o_vec1.length
					o_vec1 = simple_vec.clone
				end
			else
				found_simple_pt = true
				simple_vec.length = o_vec1.length
				o_vec1 = simple_vec.clone
			end
			@@t2 += (Time.now() - time)
			if !found_simple_pt
				origin_edges.each {|edgeID|
					next if fold_edge_id == edgeID
					edge_uv_pt1	= origin_uvhelp.get_front_UVQ(@@edge_objects[edgeID].start.position)
					edge_uv_pt2 = origin_uvhelp.get_front_UVQ(@@edge_objects[edgeID].end.position)
					edge_uv_segment = [edge_uv_pt1, edge_uv_pt2]
					
					intersection_u = self.intersect_line_line_segment_2d(trans_uv_line,edge_uv_segment)
					next if !intersection_u
					intersection_array << intersection_u
				}
				@@t3 += (Time.now() - time)
				if intersection_array.length <= 1
					msg = "Critical Error: Normalize Toolkit encountered an illegal face (a face with only two vertices).\nYour model probably has errors?"
					msg += "To check, select the menu item 'Window' -> Model Info, then select 'Statictics' and click on 'Fix Problems'."
					self.alert_error(msg,true)
					Sketchup.send_action CMD_SELECT
					return false			
				end #if
				
				intersection_array.sort!									#ok lets find if the vector is pointing the right way
				reverse = (intersection_array.index(0) % 2) ? 1 : -1
				o_vec1.reverse! if reverse == -1
			end #if
			o_pt1 = fold_pt2 + o_vec1
			o_vec2 	= origin_face.normal								#make the origin face's normal an orientation vector
																		#we need two vectors at right angles to get around the
																		#angle_between not greater than 180 degrees problem.
			smallest_angle 	= 360.degrees
			@@t4 += (Time.now() - time)
			possible_face_ids.each {|faceID|
				next if !@@faces_to_verts.has_key?(faceID)
				next if faceID == origin_face_id
				
				trans 			= Geom::Transformation.rotation fold_pt2, @@face_objects[faceID].normal, reverse * 90.degrees
				p_vec1 			= fold_vec.transform trans
				
				pface_uvhelp 	= @@face_objects[faceID].get_UVHelper true, false, @@tw		#UVHelper can give us 2d coordinates for a 3d face :)
				fold_uv_pt1		= pface_uvhelp.get_front_UVQ(fold_pt1)						#construct the corresponding uv points that are uv
				fold_uv_pt2		= pface_uvhelp.get_front_UVQ(fold_pt2)						#representations of the 3d coordinates
				fold_uv_pt3		= pface_uvhelp.get_front_UVQ(fold_pt3)
				fold_uv_vec		= fold_uv_pt1.vector_to fold_uv_pt3
				
				trans_uv_vec	= Geom::Vector3d.new(fold_uv_vec[1],-fold_uv_vec[0],1)		#construct a fold edge normal line to
				trans_uv_pt		= fold_uv_pt2 + trans_uv_vec								#calculate the line segments of intersection
				trans_uv_line	= [fold_uv_pt2, trans_uv_pt]
				
				simple_vec = p_vec1.clone
				simple_vec.length = 0.001
				simple_trans_pt = fold_pt2 + simple_vec
				found_simple_pt = false
				reverse = 1
				if @@face_objects[faceID].classify_point(simple_trans_pt) == Sketchup::Face::PointOutside
					simple_vec.reverse!
					simple_trans_pt = fold_pt2 + simple_vec
					if @@face_objects[faceID].classify_point(simple_trans_pt) != Sketchup::Face::PointOutside
						reverse = -1
						found_simple_pt = true
						simple_vec.length = p_vec1.length
						p_vec1 = simple_vec.clone
					end
				else
					found_simple_pt = true
					simple_vec.length = p_vec1.length
					p_vec1 = simple_vec.clone
				end
				
				if !found_simple_pt
					@@face_objects[faceID].each {|edgeID|
						next if fold_edge_id == edgeID
						edge_uv_pt1	= pface_uvhelp.get_front_UVQ(@@edge_objects[edgeID].start.position)
						edge_uv_pt2 = pface_uvhelp.get_front_UVQ(@@edge_objects[edgeID].end.position)
						edge_uv_segment = [edge_uv_pt1, edge_uv_pt2]
						
						intersection_u = self.intersect_line_line_segment_2d(trans_uv_line,edge_uv_segment)
						next if !intersection_u
						intersection_array << intersection_u
					}
					@@t3 += (Time.now() - time)
					if intersection_array.length <= 1
						msg = "Critical Error: Normalize Toolkit encountered an illegal face (a face with only two vertices).\nYour model probably has errors?"
						msg += "To check, select the menu item 'Window' -> Model Info, then select 'Statictics' and click on 'Fix Problems'."
						self.alert_error(msg,true)
						Sketchup.send_action CMD_SELECT
						return false			
					end #if
					
					intersection_array.sort!									#ok lets find if the vector is pointing the right way
					reverse = (intersection_array.index(0) % 2) ? 1 : -1
					p_vec1.reverse! if reverse == -1
				end #if
				
				o_ang1 = o_vec1.angle_between p_vec1
				o_ang2 = o_vec2.angle_between p_vec1
				
				manifold_angle = (o_ang2 <= 90.degrees) ? o_ang1 : 180.degrees + (180.degrees - o_ang1)

				if manifold_angle < smallest_angle
					smallest_angle = manifold_angle
					manifold_face = @@face_objects[faceID]
				end
			}
			@@t5 += (Time.now() - time)
			if !manifold_face
				return false
			end
			
			if check_orientation
				direction = self.get_rotation_direction(fold_vec,fold_pt2,origin_face.normal,o_pt1)				#work out whether the vector is pointing in the right direction
				trans 	= Geom::Transformation.rotation fold_pt2, fold_vec, (smallest_angle * direction)		#rotate the origin normal so its aligned with
				m_normal 	= o_vec2.transform trans															#the manifold's normal. This should give a vector
				m_ang1		= m_normal.angle_between manifold_face.normal										#that angles 180 degrees from the manifold's normal
				oriented = (m_ang1 > 90.degrees) ? true : false													#if its less than 90 degrees, reverse the face.
				#puts "smallest angle: "+smallest_angle.to_s
				return Hash[
					"manifold_face" => manifold_face,
					"oriented"	=> oriented
				]
			end #if
			@@t6 += (Time.now() - time)
			
			return manifold_face
		end #def

		
		def get_rotation_direction(axis_vec,axis_point,rotation_vector,rotation_point)
			trans = Geom::Transformation.rotation axis_point, axis_vec, 90.degrees
			point = rotation_point.transform trans
			vec = axis_point.vector_to point
			
			angle = rotation_vector.angle_between vec
			direction = (angle > 90.degrees) ? -1 : 1
			return direction
		end #def
		
		def deactivate(view)
			puts "tool deactivated"
			view.invalidate
			Sketchup.active_model.selection.clear
			@@entityID = false
		end #def
		
		def suspend(view)
			#puts "tool suspended"
			view.invalidate
		end #def

		def onSetCursor
			UI.set_cursor(@@cursor_active)
		end #def
		
		def draw(view)
			v_height = view.vpheight
			pt = Geom::Point3d.new 13,(v_height-145),0
			status = view.draw_text pt, "Statistics:"
			pt = Geom::Point3d.new 13,(v_height-130),0
			status = view.draw_text pt, "Faces Found: "+@@faces_to_edges.length.to_s
			pt = Geom::Point3d.new 13,(v_height-115),0
			status = view.draw_text pt, "Faces In Surface: "+@@vt_faces_in_surface_length
			pt = Geom::Point3d.new 13,(v_height-100),0
			status = view.draw_text pt, "Border Edges Found: "+@@vt_border_edge_length
			pt = Geom::Point3d.new 13,(v_height-85),0
			status = view.draw_text pt, "Non-Manifold Faces Found: "+@@vt_non_manifold_faces_length
			pt = Geom::Point3d.new 13,(v_height-70),0
			status = view.draw_text pt, "Time Taken: "+@@vt_time_taken
			pt = Geom::Point3d.new 13,(v_height-55),0
			status = view.draw_text pt, "Legend:"
			pt = Geom::Point3d.new 13,(v_height-40),0
			status = view.draw_text pt, "Selected Faces: 2d manifold errors"
			pt = Geom::Point3d.new 13,(v_height-25),0
			status = view.draw_text pt, "Selected Edges: Border edge of surface"
			Sketchup::set_status_text("Select a 2d manifold face within your previous selection",SB_PROMPT)
		end #def
		
	end #class

end #module

#############################################
# this part adds the plugin option to the plugin menu
if not file_loaded?(File.basename(__FILE__))
  
	submenu = UI.menu("Tools").add_submenu("Normalize Toolkit")
	cmd = UI::Command.new("Test manifold algorithm") {
		@@tool = NormalizeToolkit::NormalizeFaces.new
		tool = Sketchup.active_model.select_tool @@tool
	}
	submenu.add_item(cmd)
	# submenu = UI.menu("Tools").add_submenu("Normalize Toolkit")
	# cmd = UI::Command.new("Normalize Single Surface") {
		# @@tool = NormalizeToolkit::NormalizeFaces.new
		# tool = Sketchup.active_model.select_tool @@tool
	# }
	# submenu.add_item(cmd)	
	# cmd = UI::Command.new("Normalize Multiple Surfaces") {
		# @@tool = NormalizeToolkit::NormalizeFaces.new
		# tool = Sketchup.active_model.select_tool @@tool
	# }
	# submenu.add_item(cmd)
end #if
file_loaded(File.basename(__FILE__))